import Vue from 'vue';
import menuMixin from '../../mixin/menu';
import { appGetters, appMutations, headerGetters } from '../../store';
import NavMenu from '../../component/NavMenu';
import LoadingSpinner from '../../component/LoadingSpinner';
import GhostMenu from './GhostMenu';
import { getRouterActiveMenu } from '../../config/logic';
import { getMenuByFullPath } from '../../store';
import { findFirstLeaf } from '../../util';
export default {
  name: 'HorizontalResizableMenu',
  mixins: [menuMixin],
  data() {
    return {
      lastVisibleIndex: undefined,
      seed: 0
    };
  },
  computed: {
    menus() {
      const {
        menus,
        navMode
      } = appGetters;
      switch (navMode) {
        case 'head':
          return menus;
        case 'mix':
          return menus.map(menu => ({
            ...menu,
            children: undefined
          }));
        default:
          return [];
      }
    },
    realMenus() {
      const {
        lastVisibleIndex,
        menus,
        seed
      } = this;
      if (lastVisibleIndex === undefined) {
        return menus;
      }
      const fullPath = `_${seed}`;
      if (lastVisibleIndex === -1) {
        return [{
          fullPath,
          meta: {
            icon: 'el-icon-menu'
          },
          children: menus
        }];
      }
      const visible = menus.slice(0, lastVisibleIndex + 1);
      const hidden = menus.slice(lastVisibleIndex + 1);
      visible.push({
        fullPath,
        meta: {
          title: '...'
        },
        children: hidden
      });
      return visible;
    },
    hasDom() {
      return !appGetters.loadingMenu && this.menus.length > 0;
    }
  },
  watch: {
    $route: {
      immediate: true,
      handler(to) {
        if (this.setActiveRootMenu(to)) {
          this.setActiveMenu(appGetters.navMode, to);
        }
      }
    },
    realMenus: {
      immediate: true,
      handler(v) {
        if (!this.hasDom) return;
        this.setActiveMenu(undefined, undefined, false);
        this.setDefaultActiveMenu(v);
      }
    },
    hasDom: {
      immediate: true,
      async handler(value) {
        await this.$nextTick();
        value ? this.startObserver() : this.stopObserver();
      }
    },
    lastVisibleIndex() {
      this.seed++;
    }
  },
  methods: {
    setActiveRootMenu({
      path,
      meta: {
        activeMenu
      }
    }) {
      const menu = getMenuByFullPath(activeMenu || path);
      if (!menu) return false;
      let rootMenu = menu;
      while (rootMenu.parent) {
        rootMenu = rootMenu.parent;
      }
      appMutations.activeRootMenu(rootMenu.fullPath);
      return true;
    },
    setActiveMenu(navMode = appGetters.navMode, route = this.$route, needReset = true) {
      if (navMode === 'mix') {
        this.activeMenu = appGetters.activeRootMenu;
      } else this.activeMenu = getRouterActiveMenu(route);
      needReset && this.resetActiveMenu();
    },
    onSelect(index) {
      if (appGetters.navMode === 'mix') {
        return this.onSelectRootMenu(index);
      }
      this.actionOnSelectMenu(index);
    },
    onSelectRootMenu(index) {
      const root = getMenuByFullPath(index);
      const {
        leaf,
        hasOtherLeaf
      } = findFirstLeaf(root);
      if (!leaf || appGetters.activeRootMenu === index && hasOtherLeaf) {
        return;
      }
      this.actionOnSelectMenu(leaf.fullPath);
    },
    getMenuEl() {
      return this.$el;
    },
    genGhostMenu() {
      if (Reflect.has(this, '$_ghostMenu')) {
        this.$_ghostMenu.menus = this.menus;
        return;
      }
      const ctor = Vue.extend(GhostMenu);
      const instance = new ctor({
        data: {
          menus: this.menus
        }
      });
      instance.$watch('menuItemSizes', value => {
        this.$menuItemSizes = value;
        this.resize();
      });
      instance.$mount();
      this.$_ghostMenu = instance;
    },
    destroyGhostMenu() {
      Reflect.has(this, '$_ghostMenu') && this.$_ghostMenu.$destroy();
    },
    setOverflowedIndicatorWidth() {
      const ul = document.createElement('ul');
      ul.className = 'el-menu--horizontal el-menu el-menu--horizontal';
      ul.style.position = 'fixed';
      ul.style.top = '-100px';
      ul.style.right = '-1000px';
      ul.innerHTML = `<li class="el-submenu"><div class="el-submenu__title"><span>...</span></div></li>`;
      document.body.appendChild(ul);
      this.$overflowedIndicatorWidth = ul.children[0].offsetWidth + 1;
      document.body.removeChild(ul);
    },
    resize(entry) {
      if (!entry) return;
      const width = this.getMenuEl().getBoundingClientRect().width;
      const {
        $menuItemSizes,
        $overflowedIndicatorWidth
      } = this;
      let lastVisibleIndex = -1;
      for (let i = 0, sum = 0; i < $menuItemSizes.length; i++) {
        sum += $menuItemSizes[i];
        if (sum + $overflowedIndicatorWidth > width) {
          break;
        }
        lastVisibleIndex += 1;
        if (lastVisibleIndex === $menuItemSizes.length - 1) {
          lastVisibleIndex = undefined;
          break;
        }
      }
      this.lastVisibleIndex = lastVisibleIndex;
    },
    startObserver() {
      this.resizeObserver = new window.ResizeObserver(this.resize);
      this.resizeObserver.observe(this.getMenuEl());
    },
    stopObserver() {
      if (this.resizeObserver) {
        this.resizeObserver.disconnect();
        delete this.resizeObserver;
      }
      this.destroyGhostMenu();
    }
  },
  created() {
    this.$menuItemSizes = [];
    this.$overflowedIndicatorWidth = 0;
    this.$watch(() => appGetters.loadingMenu, loading => {
      !loading && this.$nextTick(() => {
        const route = this.$route;
        if (this.setActiveRootMenu(route)) {
          this.setActiveMenu(appGetters.navMode, route);
        }
      });
    });
  },
  mounted() {
    this.$nextTick(() => {
      this.setOverflowedIndicatorWidth();
    });
  },
  beforeDestroy() {
    this.stopObserver();
  },
  render() {
    const h = arguments[0];
    if (appGetters.loadingMenu) {
      return h("div", {
        "style": "position: relative;width: 100%;height: 100%"
      }, [h(LoadingSpinner)]);
    }
    if (this.menus.length === 0) return;
    this.genGhostMenu();
    return h(NavMenu, {
      "ref": "nav-menu",
      "class": "horizontal-menu",
      "attrs": {
        "menus": this.realMenus,
        "theme": headerGetters.theme,
        "mode": "horizontal",
        "default-active": this.defaultActive,
        "show-collapse-icon": headerGetters.showCollapseIcon,
        "menu-icon-slot": headerGetters.menuIconSlot,
        "menu-content-slot": headerGetters.menuContentSlot
      },
      "on": {
        "select": this.onSelect
      }
    });
  }
};