import { appGetters, pageGetters, pageMutations, tagsViewGetters, tagsViewMutations } from '../../store';
import useContextMenu from '../../component/ContextMenu/functionalUse';
import HorizontalScroller from '../../component/HorizontalScroller';
import { refreshPage } from '../../helper';
import { getRouterKey, getRouterTitle, isRedirectRouter } from '../../config/logic';
import { getAffixTagsFromMenuTree } from './util';
export default {
  name: 'TagsView',
  data() {
    return {
      activeKey: '',
      selectedTag: undefined
    };
  },
  computed: {
    contextMenuItems() {
      const {
        visitedViews
      } = tagsViewGetters;
      const closeable = visitedViews.length > 1 && !this.selectedTag.fixed;
      return [{
        content: '刷新',
        click: this.refreshSelectedTag
      }, {
        content: closeable && '关闭',
        click: this.closeSelectedTag
      }, {
        content: '关闭其他',
        click: this.closeOthersTags
      }, {
        content: '关闭全部',
        click: this.closeAllTags
      }].filter(i => i.content);
    },
    tagsSlot() {
      return this._u([{
        key: 'default',
        fn: this.renderTags,
        proxy: true
      }]);
    }
  },
  watch: {
    $route(to, from) {
      tagsViewGetters.enableChangeTransition && this.decideRouteTransition(to, from);
      if (isRedirectRouter(to)) return;
      this.setActiveKey(to);
      this.addTag(to);
      this.$nextTick(this.moveToCurrentTag);
    }
  },
  methods: {
    setActiveKey(route) {
      this.activeKey = getRouterKey(route);
    },
    decideRouteTransition(to, from) {
      const {
        next,
        prev
      } = pageGetters.transition;
      const {
        visitedViews
      } = tagsViewGetters;
      const fromKey = getRouterKey(from);
      const toKey = getRouterKey(to);
      const fromIndex = visitedViews.findIndex(i => i.key === fromKey);
      const toIndex = visitedViews.findIndex(i => i.key === toKey);
      let transitionName = prev;
      if (toIndex === -1 || fromIndex < toIndex) {
        transitionName = next;
      }
      pageMutations.transition({
        curr: transitionName
      });
    },
    addTag(route) {
      tagsViewMutations.addTagAndCache({
        ...route,
        meta: {
          ...route.meta,
          title: getRouterTitle(route)
        }
      });
    },
    moveToCurrentTag() {
      const {
        scroller
      } = this.$refs;
      const cur = Array.from(scroller.$el.children).find(el => el.classList.contains('active'));
      cur && scroller.moveToTarget(cur);
    },
    gotoLastTag(refresh = false, replace = true) {
      const views = tagsViewGetters.visitedViews;
      const router = this.$router;
      const pushMethod = replace ? 'replace' : 'push';
      if (views.length === 0) {
        router[pushMethod]('/');
        return;
      }
      const latest = views[views.length - 1];
      this.activeKey === latest.key ? refresh && refreshPage(router) : this.$nextTick(() => router[pushMethod](latest));
    },
    refreshSelectedTag(view = this.selectedTag) {
      refreshPage(this.$router, view);
    },
    closeSelectedTag(view = this.selectedTag) {
      if (tagsViewGetters.visitedViews.length <= 1 || view.fixed) return;
      tagsViewMutations.delTagAndCache(view);
      this.activeKey === view.key && this.gotoLastTag();
    },
    closeOthersTags() {
      tagsViewMutations.delOtherTagAndCache(this.selectedTag);
      this.gotoLastTag();
    },
    closeAllTags() {
      tagsViewMutations.delAllTagAndCache();
      this.gotoLastTag(true);
    },
    openContextMenu(tag, e) {
      e.preventDefault();
      this.$contextmenu && this.$contextmenu();
      this.selectedTag = tag;
      this.$contextmenu = useContextMenu(this.contextMenuItems, {
        left: e.clientX + 15,
        top: e.clientY + 5
      });
    },
    renderDefaultStyleTag(h, {
      key,
      active = false,
      on,
      title,
      close
    }) {
      const className = `tags-view-item${active ? ' active' : ''}`;
      return h("div", {
        "key": key,
        "class": className,
        "on": {
          ...on
        }
      }, [h("div", {
        "class": "tags-view-item__dot"
      }), h("span", [title]), close && h("i", {
        "class": "el-icon-close",
        "on": {
          "click": close
        }
      })]);
    },
    renderTags() {
      const {
        $createElement: h,
        $router,
        activeKey
      } = this;
      const {
        itemSlot,
        visitedViews
      } = tagsViewGetters;
      const renderFn = itemSlot || this.renderDefaultStyleTag;
      return visitedViews.map(view => {
        const active = activeKey === view.key;
        const showClose = !view.fixed && visitedViews.length > 1;
        const on = {
          contextmenu: e => this.openContextMenu(view, e)
        };
        const onClose = e => {
          e.stopPropagation();
          this.closeSelectedTag(view);
        };
        if (!active) {
          on.click = () => $router.push(view, () => undefined);
        }
        return renderFn(h, {
          key: view.key,
          active,
          on,
          title: view.meta.title,
          close: showClose && onClose
        });
      });
    }
  },
  mounted() {
    this.setActiveKey(this.$route);
    this.$watch(() => appGetters.menus, menus => {
      const fixedTags = getAffixTagsFromMenuTree(this.$router, menus);
      fixedTags.forEach(tag => tagsViewMutations.addTagOnly(tag, true));
    }, {
      immediate: true
    });
    this.addTag(this.$route);
  },
  render() {
    const h = arguments[0];
    return h("nav", {
      "class": "tags-view"
    }, [h(HorizontalScroller, {
      "ref": "scroller",
      "scopedSlots": this.tagsSlot
    })]);
  }
};